"""Script to compute contact maps from Chains."""
import csv
import pathlib
import sys
import time

import numpy as np
from Bio import SeqIO

from pyge.contacts import contactmap
from read_modres import read_modres


if len(sys.argv) != 4:
    print("Usage: python 1_cm_computation.py <starting_idx> <ending_idx/'end'> <id>")
    sys.exit(1)
elif len(sys.argv) > 1:
    batch_start = int(sys.argv[1])
    if sys.argv[2] == 'end':
        batch_end = 'end'
    else:
        batch_end = int(sys.argv[2])
    out_id = sys.argv[3]

membrane_dir = pathlib.Path("/mnt/MEMBRANE/")
input_chains_dir = membrane_dir / "CHAINS"
alignments_dir = membrane_dir / "ALIGNMENTS"
cm_dir = membrane_dir / "CM"
cm_log_dir = cm_dir / "LOG"

with open(membrane_dir / "chains_alis.csv", "r", encoding="utf-8") as in_file:
    reader = csv.reader(in_file, delimiter=",")
    # skip header
    next(reader)
    next(reader)
    chains = []
    for row in reader:
        if len(row) == 3:
            chains.append((row[0], pathlib.Path(row[1]), pathlib.Path(row[2])))
        elif len(row) == 2:
            chains.append((row[0], pathlib.Path(row[1])))
        else:
            raise ValueError(f"Chain {row[0]} has {len(row)} paths")
assert len(chains) == 1282

# save those chains with modres
chains_changed_file = open(cm_log_dir / f"chains_modres_{out_id}.csv", "w")
writer = csv.writer(chains_changed_file)
writer.writerow(["# generated by remove_hetatm.py"])
writer.writerow(["# chain_id", "non_std_aa"])

# save those chains with modres
err_file = open(cm_log_dir / f"cm_err_log_{out_id}.csv", "w")
err_writer = csv.writer(err_file)
err_writer.writerow(["# generated by remove_hetatm.py"])
err_writer.writerow(["# chain_id", "err_msg"])

if batch_end == 'end':
    batch_end = len(chains)
for idx_ch, chain in enumerate(chains[batch_start:batch_end]):
    print(f"====== Name: {chain[0]}; idx_ch: {idx_ch} ======")
    start = time.time()

    chain_path = chain[1]
    chain_name = chain_path.name[-5:-4]
    
    # search for MODRES to be included
    modres = read_modres(chain_path, chain_name, list_only=True)
    if not modres:
        modres = ["UNK"]
    else:
        print(f"MODRES found: {modres}")
        modres = modres + ["UNK"]

    if len(chain) == 3:
        print("MISRES found")
        ali_path = chain[2]
        # read sequence from alignment file
        sequence = SeqIO.to_dict(
            SeqIO.parse(ali_path, "pir")
        )[chain_path.name[-10:-4]].seq
        sequence_list = list(sequence)

        try:
            cm = contactmap.compute_contactmap(
                str(chain_path),
                model_id=1,
                chain_id=chain_name,
                threshold=4.5,
                to_include=modres,
                to_ignore=["HETATM"],
                sequence=sequence_list,
            )
        except Exception as err:
            print("ERROR")
            err_writer.writerow([chain[0], str(err)])
            continue
    elif len(chain) == 2:
        # read sequence from PDB for a check
        # This is an independent check given by the SEQRES record
        #  of the PDB file
        for record in SeqIO.parse(chain_path, "pdb-seqres"):
            if record.id[-1:] == chain_name:
                sequence_list = list(str(record.seq))
                break
        try:
            cm = contactmap.compute_contactmap(
                str(chain_path),
                model_id=1,
                chain_id=chain_name,
                threshold=4.5,
                to_include=modres,
                to_ignore=["HETATM"]
            )
        except Exception as err:
            print("ERROR")
            err_writer.writerow([chain[0], str(err)])
            continue
    else:
        raise ValueError(f"Error reading: {chain}")

    # In both cases, these two should be equal
    if len(cm) != len(sequence_list):
        print("ERROR")
        err_writer.writerow(
            [
                chain[0],
                f"CM and Seq len differs: CM: {cm.shape}; SEQ: {len(sequence_list)}"
            ]
        )
        continue
    
    # If everything is ok, then write the CM and out info files
    if modres:
        writer.writerow([chain[0], modres])

    np.savez_compressed(
        cm_dir / chain_path.name.replace(".pdb", ""), arr=cm
    )
    print(f"Res len (CM's dim 0): {len(cm)}")
    print(f"Time: {time.time() - start} s")

chains_changed_file.close()
