"""Read MODRES records from a PDB file."""


def read_modres(pdb_path, chain_id, list_only=False):
    """Read MODRES records from a PDB file.
    
    Parameters
    ----------
    pdb_path : Path, str
        Path to the PDB file.
    chain_id : str
        Chain ID to read.
    list_only : bool, optional
        If True, only return a list of modified residue names.

    Returns
    -------
    modres : dict
        Dictionary of modified residues.
        Keys are the modified residue names, and values are dictionaries
        with the standard residue name ("std")
        and a list of residue numbers ("res_num").
    """
    modres = {}
    with open(pdb_path) as pdb_file:
        for line in pdb_file:
            if line.startswith('MODRES'):
                modified_res_name = line[12:15]
                modified_chain = line[16]
                modified_res_num = line[18:22]
                standard_res = line[24:27]

                if modified_chain == chain_id:
                    if modified_res_name not in modres:
                        modres[modified_res_name] = {
                            'std': standard_res,
                            "res_num": [modified_res_num.strip(" ")]
                        }
                    else:
                        assert modres[modified_res_name]["std"] == standard_res
                        modres[modified_res_name]["res_num"].append(
                            modified_res_num.strip(" ")
                        )
            elif line.startswith('ATOM'):
                break
    if list_only:
        return list(modres.keys())
    return modres
